-- alerter allows users to set key phrases to watch for in text and then generates a large, flashy message to warn them
-- todo:
-- allow the Alert Message to contain wildcards that match part of the original message


import "GaranStuff.Alerter.Class";
import "GaranStuff.Alerter.VindarPatch";
import "Turbine";
import "Turbine.Gameplay";
import "Turbine.UI";
import "Turbine.UI.Lotro";
import "GaranStuff.Alerter.Table";
import "GaranStuff.Alerter.Strings";
import "GaranStuff.Alerter.ColorPicker";
import "GaranStuff.Alerter.DropDownList";

FlasherFont=Turbine.UI.Lotro.Font.TrajanProBold36;
FlasherColor=Turbine.UI.Color(1,0,0);
AlerterLeft=-1;
AlerterTop=-1;
alertHUDState=true;
alertMainState=false;

fontFace=Turbine.UI.Lotro.Font.Verdana16;
trimColor=Turbine.UI.Color(.3,.3,.3);
fontColor=Turbine.UI.Color(1,1,1);
backColor=Turbine.UI.Color(0,0,0);
listTextColor=Turbine.UI.Color(.9,.9,.9);

language=1;
locale = "en";
if Turbine.Shell.IsCommand("hilfe") then
	locale = "de";
	language=3;
elseif Turbine.Shell.IsCommand("aide") then
	locale = "fr";
	language=2;
end
euroFormat=(tonumber("1,000")==1);

string.split=function(str,separator)
	local tmpArray={};
	local tmpElem;
	str=tostring(str);
	if string.find("^$()%.[]*+-?",separator,1,true)~=nil then separator="%"..separator end
	for tmpElem in string.gmatch(str,"[^"..separator.."]*") do
		table.insert(tmpArray,tmpElem);
	end
	local tmpIndex=1;
	while tmpIndex<=#tmpArray do
		if tmpIndex==#tmpArray and tmpArray[tmpIndex]=="" then
			table.remove(tmpArray,tmpIndex);
		else
			if tmpArray[tmpIndex]=="" and tmpArray[tmpIndex+1]~="" then
				table.remove(tmpArray,tmpIndex);
			else
				tmpIndex=tmpIndex+1;
			end
		end
	end
	return tmpArray;
end
string.ltrim=function(str)
	return (string.gsub(str, "^%s*(.-)", "%1"))
end
string.rtrim=function(str)
	return (string.gsub(str, "^(.-)%s*$", "%1"))
end
string.trim=function(str)
	return (string.gsub(str, "^%s*(.-)%s*$", "%1"))
end

function UnloadPlugin()
	-- have to remove all shell commands before unloading or we get a crash to desktop :(
	alertMain:RemoveShellCommands();
	SaveData();
	Turbine.Chat.Received=nil;
end

function SaveData()
	local settings={};
--	settings[#settings+1]={"MapHeight",mapWindow:GetHeight()};
	PatchDataSave( Turbine.DataScope.Character, "Alerter_Settings", Settings );
	PatchDataSave( Turbine.DataScope.Character, "Alerter_Alerts", Alerts );
end
function LoadData()
	local tmpIndex;
	Alerts=PatchDataLoad( Turbine.DataScope.Character, "Alerter_Alerts");
	if Alerts==nil then Alerts={} end
	for tmpIndex=1,#Alerts do
		if Alerts[tmpIndex][3]~=nil then
			if euroFormat then
				-- replace "." with "," tonumber(string.gsub(tmpSettings[index][2].R,".",","))
				Alerts[tmpIndex][3]=tonumber((string.gsub(Alerts[tmpIndex][3],"%.",",")));
			else
				Alerts[tmpIndex][3]=tonumber((string.gsub(Alerts[tmpIndex][3],",",".")));
			end
		end
		if Alerts[tmpIndex][4]~=nil then
			if euroFormat then
				-- replace "." with "," tonumber(string.gsub(tmpSettings[index][2].R,".",","))
				Alerts[tmpIndex][4]=Turbine.UI.Color(tonumber((string.gsub(Alerts[tmpIndex][4].R,"%.",","))),tonumber((string.gsub(Alerts[tmpIndex][4].G,"%.",","))),tonumber((string.gsub(Alerts[tmpIndex][4].B,"%.",","))));
			else
				Alerts[tmpIndex][4]=Turbine.UI.Color(tonumber((string.gsub(Alerts[tmpIndex][4].R,",","."))),tonumber((string.gsub(Alerts[tmpIndex][4].G,",","."))),tonumber((string.gsub(Alerts[tmpIndex][4].B,",","."))));
			end
		end
		if Alerts[tmpIndex][6]~=nil then
			if euroFormat then
				-- replace "." with "," tonumber(string.gsub(tmpSettings[index][2].R,".",","))
				Alerts[tmpIndex][6]=tonumber((string.gsub(Alerts[tmpIndex][6],"%.",",")));
			else
				Alerts[tmpIndex][6]=tonumber((string.gsub(Alerts[tmpIndex][6],",",".")));
			end
		end
	end
end
LoadData();


Flasher=class(Turbine.UI.Window);

function Flasher:Constructor()
	Turbine.UI.Window.Constructor( self );
	self:SetMouseVisible(false);
	self:SetSize(Turbine.UI.Display:GetWidth(),Turbine.UI.Display:GetHeight());
	self.Opacity=.5;
	self.Message=Turbine.UI.Label();
	self.Message:SetParent(self);
	self.Message:SetSize(self:GetWidth(),self:GetHeight());
	self.Message:SetFont(FlasherFont);
	self.Message:SetForeColor(FlasherColor);
	self.Message:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter);
	self.Duration=1;
	self.Interval=.3;

	self:SetZOrder(1000);
	self.EndTime=0;
	self.SetAlert=function(msg, duration, color, font, interval)
		if color~=nil then
			self.Message:SetForeColor(color);
		else
			self.Message:SetForeColor(FlasherColor);
		end
		if interval~=nil then
			self.Interval=interval;
		else
			self.Interval=.3;
		end
		if font~=nil then
			self.Message:SetFont(font);
		else
			self.Message:SetFont(FlasherFont);
		end
		if duration==nil then
			duration=1;
		end
		self.Message:SetText(msg);
		self.EndTime=Turbine.Engine.GetGameTime()+duration;
		if self.Interval==0 then
			self.FlashTime=self.EndTime
		else
			self.FlashTime=Turbine.Engine.GetGameTime()+self.Interval/2;
		end
		self:SetVisible(true);
		self:SetWantsUpdates(true);
	end
	self.Update=function()
		if Turbine.Engine.GetGameTime()>=self.EndTime then
			self:SetWantsUpdates(false);
			self:SetVisible(false);
			self.FlashTime=0;
		elseif Turbine.Engine.GetGameTime()>=self.FlashTime then
			self:SetVisible(not self:IsVisible());
			self.FlashTime=Turbine.Engine.GetGameTime()+self.Interval/2;
		end
	end
end

Alerter=class(Turbine.UI.Lotro.Window);
function Alerter:Constructor()
	local tmpIndex;
	Turbine.UI.Lotro.Window.Constructor( self );
	self:SetSize(500,500);
	if AlerterLeft<0 then AlerterLeft=(Turbine.UI.Display:GetWidth()-self:GetWidth())/2 end
	if AlerterTop<0 then AlerterTop=(Turbine.UI.Display:GetHeight()-self:GetHeight())/2 end
	self:SetPosition(AlerterLeft,AlerterTop);
	self:SetText(Resource[language][2])
	self.Flasher=Flasher();

	self.AlertCaption=Turbine.UI.Label();
	self.AlertCaption:SetParent(self);
	self.AlertCaption:SetSize(100,20);
	self.AlertCaption:SetPosition(10,45);
	self.AlertCaption:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft);
	self.AlertCaption:SetText(Resource[language][3]..":");
	self.AlertSelect=DropDownList();
	self.AlertSelect:SetParent(self);
	self.AlertSelect:SetSize(180,20);
	self.AlertSelect:SetPosition(self.AlertCaption:GetLeft()+self.AlertCaption:GetWidth()+2,self.AlertCaption:GetTop());
	self.AlertSelect:SetBorderColor(trimColor);
	self.AlertSelect:SetBackColor(backColor);
	self.AlertSelect.CurrentValue:SetBackColor(backColor);
 	self.AlertSelect:SetTextColor(listTextColor);
	self.AlertSelect:SetDropRows(3);
	self.AlertSelect:SetZOrder(1);
	self.AlertSelect.SelectedIndexChanged=function()
		local tmpAlertIndex=self.AlertSelect:GetValue();
		if tmpAlertIndex>0 then
			local tmpSettingIndex,tmpIndex, tmpFont;
			if Alerts[tmpAlertIndex][1]==nil then
				self.LabelText:SetText("");
			else
				self.LabelText:SetText(Alerts[tmpAlertIndex][1]);
			end
			if Alerts[tmpAlertIndex][2]==nil then
				self.MessageText:SetText("");
			else
				self.MessageText:SetText(Alerts[tmpAlertIndex][2]);
			end
			if Alerts[tmpAlertIndex][3]==nil then
				self.DurationText:SetText("");
			else
				self.DurationText:SetText(Alerts[tmpAlertIndex][3]);
			end
			if Alerts[tmpAlertIndex][4]==nil then
				self.MessageColor:SetColor(FlasherColor);
			else
				self.MessageColor:SetColor(Alerts[tmpAlertIndex][4]);
			end
			tmpSettingIndex=24;
			tmpFont=Alerts[tmpAlertIndex][5];
			if tmpFont==nil then
				tmpFont=FlasherFont;
			end
			for tmpIndex=1,self.FontSelect.ListData:GetItemCount() do
				if self.FontSelect.ListData:GetItem(tmpIndex).DataValue==tmpFont then
					tmpSettingIndex=tmpIndex;
					break;
				end
			end
			self.FontSelect:SetSelectedIndex(tmpSettingIndex);
			if Alerts[tmpAlertIndex][6]==nil then
				self.IntervalText:SetText("");
			else
				self.IntervalText:SetText(Alerts[tmpAlertIndex][6]);
			end
			local m,tmpVal;
			if Alerts[tmpAlertIndex][7]==nil or type(Alerts[tmpAlertIndex][7])~="table" then
				for tmpIndex=1,#self.ChannelSelect do
					self.ChannelSelect[tmpIndex]:SetChecked(false);
				end
			else
				for tmpIndex=1,#self.ChannelSelect do
					m,tmpVal=math.frexp(self.ChannelSelect[tmpIndex].Value);
					if Alerts[tmpAlertIndex][7][tmpVal]==nil then
						self.ChannelSelect[tmpIndex]:SetChecked(false);
					else
						self.ChannelSelect[tmpIndex]:SetChecked(Alerts[tmpAlertIndex][7][tmpVal]);
					end
				end
			end
			if Alerts[tmpAlertIndex][8]==nil then
				self.PatternText:SetText("");
			else
				self.PatternText:SetText(Alerts[tmpAlertIndex][8]);
			end
		end
	end

	self.AlertSelect:AddItem(Resource[language][4],0);
	for tmpIndex=1,#Alerts do
		self.AlertSelect:AddItem(Alerts[tmpIndex][1],tmpIndex);
	end

	self.LabelCaption=Turbine.UI.Label();
	self.LabelCaption:SetParent(self);
	self.LabelCaption:SetSize(100,20);
	self.LabelCaption:SetPosition(10,self.AlertCaption:GetTop()+25);
	self.LabelCaption:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft);
	self.LabelCaption:SetText(Resource[language][5]..":");
	self.LabelText=Turbine.UI.Lotro.TextBox();
	self.LabelText:SetParent(self);
	self.LabelText:SetSize(200,20);
	self.LabelText:SetFont(fontFace);
	self.LabelText:SetPosition(self.LabelCaption:GetLeft()+self.LabelCaption:GetWidth()+2,self.LabelCaption:GetTop());

	self.MessageCaption=Turbine.UI.Label();
	self.MessageCaption:SetParent(self);
	self.MessageCaption:SetSize(100,20);
	self.MessageCaption:SetPosition(10,self.LabelCaption:GetTop()+25);
	self.MessageCaption:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft);
	self.MessageCaption:SetText(Resource[language][6]..":");
	self.MessageText=Turbine.UI.Lotro.TextBox();
	self.MessageText:SetParent(self);
	self.MessageText:SetSize(self:GetWidth()-self.MessageCaption:GetLeft()-self.MessageCaption:GetWidth()-12,20);
	self.MessageText:SetFont(fontFace);
	self.MessageText:SetPosition(self.MessageCaption:GetLeft()+self.MessageCaption:GetWidth()+2,self.MessageCaption:GetTop());

	self.DurationCaption=Turbine.UI.Label();
	self.DurationCaption:SetParent(self);
	self.DurationCaption:SetSize(100,20);
	self.DurationCaption:SetPosition(10,self.MessageCaption:GetTop()+25);
	self.DurationCaption:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft);
	self.DurationCaption:SetText(Resource[language][9]..":");
	self.DurationText=Turbine.UI.Lotro.TextBox();
	self.DurationText:SetParent(self);
	self.DurationText:SetSize(80,20);
	self.DurationText:SetFont(fontFace);
	self.DurationText:SetPosition(self.DurationCaption:GetLeft()+self.DurationCaption:GetWidth()+2,self.DurationCaption:GetTop());

	self.ColorCaption=Turbine.UI.Label();
	self.ColorCaption:SetParent(self);
	self.ColorCaption:SetSize(100,20);
	self.ColorCaption:SetPosition(10,self.DurationCaption:GetTop()+25);
	self.ColorCaption:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft);
	self.ColorCaption:SetText(Resource[language][7]..":");

	self.MessageColor=ColorPicker();
	self.MessageColor:SetParent(self);
	self.MessageColor:SetPosition(self.ColorCaption:GetLeft()+self.ColorCaption:GetWidth()+2,self.ColorCaption:GetTop()-1);
	self.MessageColor:SetTrimColor(Turbine.UI.Color(.4,.4,.4));
	self.MessageColor:SetColor(Turbine.UI.Color(1,0,0));
--	self.MessageColor.ColorChanged=function()
--		self:SetMessageColor(self.MessageColor:GetColor());
--	end

	self.FontCaption=Turbine.UI.Label();
	self.FontCaption:SetParent(self);
	self.FontCaption:SetSize(100,20);
	self.FontCaption:SetPosition(10,self.ColorCaption:GetTop()+25);
	self.FontCaption:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft);
	self.FontCaption:SetText(Resource[language][3]..":");
	self.FontSelect=DropDownList();
	self.FontSelect:SetParent(self);
	self.FontSelect:SetSize(180,20);
	self.FontSelect:SetPosition(self.FontCaption:GetLeft()+self.FontCaption:GetWidth()+2,self.FontCaption:GetTop());
	self.FontSelect:SetBorderColor(trimColor);
	self.FontSelect:SetBackColor(backColor);
	self.FontSelect.CurrentValue:SetBackColor(backColor);
 	self.FontSelect:SetTextColor(listTextColor);
	self.FontSelect:SetDropRows(3);
	self.FontSelect:SetZOrder(1);
	self.FontSelect.SelectedIndexChanged=function()
	end
	self.FontSelect:AddItem("Arial 12",Turbine.UI.Lotro.Font.Arial12);
	self.FontSelect:AddItem("Calibri 14",0x4200002b);
	self.FontSelect:AddItem("TimesRoman 12",0x42000021);
	self.FontSelect:AddItem("TimesRoman 20",0x4200000e);
	self.FontSelect:AddItem("TimesRoman 22",0x4200000f);
	self.FontSelect:AddItem("TimesRoman 24",0x42000010);
	self.FontSelect:AddItem("TrajanPro 13",Turbine.UI.Lotro.Font.TrajanPro13);
	self.FontSelect:AddItem("TrajanPro 14",Turbine.UI.Lotro.Font.TrajanPro14);
	self.FontSelect:AddItem("TrajanPro 15",Turbine.UI.Lotro.Font.TrajanPro15);
	self.FontSelect:AddItem("TrajanPro 16",Turbine.UI.Lotro.Font.TrajanPro16);
	self.FontSelect:AddItem("TrajanPro 18",Turbine.UI.Lotro.Font.TrajanPro18);
	self.FontSelect:AddItem("TrajanPro 19",Turbine.UI.Lotro.Font.TrajanPro19);
	self.FontSelect:AddItem("TrajanPro 20",Turbine.UI.Lotro.Font.TrajanPro20);
	self.FontSelect:AddItem("TrajanPro 21",Turbine.UI.Lotro.Font.TrajanPro21);
	self.FontSelect:AddItem("TrajanPro 23",Turbine.UI.Lotro.Font.TrajanPro23);
	self.FontSelect:AddItem("TrajanPro 24",Turbine.UI.Lotro.Font.TrajanPro24);
--	self.FontSelect:AddItem("TrajanPro 25",Turbine.UI.Lotro.Font.TrajanPro25);
	self.FontSelect:AddItem("TrajanPro 26",Turbine.UI.Lotro.Font.TrajanPro26);
	self.FontSelect:AddItem("TrajanPro 28",Turbine.UI.Lotro.Font.TrajanPro28);
	self.FontSelect:AddItem("TrajanProBold 16",Turbine.UI.Lotro.Font.TrajanProBold16);
	self.FontSelect:AddItem("TrajanProBold 22",Turbine.UI.Lotro.Font.TrajanProBold22);
	self.FontSelect:AddItem("TrajanProBold 24",Turbine.UI.Lotro.Font.TrajanProBold24);
	self.FontSelect:AddItem("TrajanProBold 25",Turbine.UI.Lotro.Font.TrajanProBold25);
	self.FontSelect:AddItem("TrajanProBold 30",Turbine.UI.Lotro.Font.TrajanProBold30);
	self.FontSelect:AddItem("TrajanProBold 36",Turbine.UI.Lotro.Font.TrajanProBold36);
	self.FontSelect:AddItem("Verdana 10",Turbine.UI.Lotro.Font.Verdana10);
	self.FontSelect:AddItem("Verdana 12",Turbine.UI.Lotro.Font.Verdana12);
	self.FontSelect:AddItem("Verdana 14",Turbine.UI.Lotro.Font.Verdana14);
	self.FontSelect:AddItem("Verdana 16",Turbine.UI.Lotro.Font.Verdana16);
	self.FontSelect:AddItem("Verdana 18",Turbine.UI.Lotro.Font.Verdana18);
	self.FontSelect:AddItem("Verdana 20",Turbine.UI.Lotro.Font.Verdana20);
	self.FontSelect:AddItem("Verdana 22",Turbine.UI.Lotro.Font.Verdana22);
	self.FontSelect:AddItem("Verdana 23",Turbine.UI.Lotro.Font.Verdana23);

	self.FontSelect:SetSelectedIndex(24); -- default to the largest

	self.IntervalCaption=Turbine.UI.Label();
	self.IntervalCaption:SetParent(self);
	self.IntervalCaption:SetSize(100,20);
	self.IntervalCaption:SetPosition(10,self.FontCaption:GetTop()+25);
	self.IntervalCaption:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft);
	self.IntervalCaption:SetText(Resource[language][10]..":");
	self.IntervalText=Turbine.UI.Lotro.TextBox();
	self.IntervalText:SetParent(self);
	self.IntervalText:SetSize(80,20);
	self.IntervalText:SetFont(fontFace);
	self.IntervalText:SetPosition(self.IntervalCaption:GetLeft()+self.IntervalCaption:GetWidth()+2,self.IntervalCaption:GetTop());

	self.ChannelCaption=Turbine.UI.Label();
	self.ChannelCaption:SetParent(self);
	self.ChannelCaption:SetSize(self:GetWidth()-20,20);
	self.ChannelCaption:SetPosition(10,self.IntervalCaption:GetTop()+25);
	self.ChannelCaption:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft);
	self.ChannelCaption:SetText(Resource[language][12]..":");

	local offset=(self:GetWidth()-20)/4;
	self.ChannelSelect={}
	self.ChannelSelect[1]=Turbine.UI.Lotro.CheckBox();
	self.ChannelSelect[1]:SetParent(self)
	self.ChannelSelect[1]:SetText(Resource[language][14]);
	self.ChannelSelect[1]:SetSize(offset-10,20);
	self.ChannelSelect[1]:SetPosition(10,self.ChannelCaption:GetTop()+25);
	self.ChannelSelect[1].Value=0
	self.ChannelSelect[2]=Turbine.UI.Lotro.CheckBox();
	self.ChannelSelect[2]:SetParent(self)
	self.ChannelSelect[2]:SetText(Resource[language][15]);
	self.ChannelSelect[2]:SetSize(offset-10,20);
	self.ChannelSelect[2]:SetPosition(offset,self.ChannelCaption:GetTop()+25);
	self.ChannelSelect[2].Value=Turbine.ChatType.Admin
	self.ChannelSelect[3]=Turbine.UI.Lotro.CheckBox();
	self.ChannelSelect[3]:SetParent(self)
	self.ChannelSelect[3]:SetText(Resource[language][16]);
	self.ChannelSelect[3]:SetSize(offset-10,20);
	self.ChannelSelect[3]:SetPosition(offset*2,self.ChannelCaption:GetTop()+25);
	self.ChannelSelect[3].Value=Turbine.ChatType.Advancement
	self.ChannelSelect[4]=Turbine.UI.Lotro.CheckBox();
	self.ChannelSelect[4]:SetParent(self)
	self.ChannelSelect[4]:SetText(Resource[language][17]);
	self.ChannelSelect[4]:SetSize(offset-10,20);
	self.ChannelSelect[4]:SetPosition(offset*3,self.ChannelCaption:GetTop()+25);
	self.ChannelSelect[4].Value=Turbine.ChatType.Advice
	self.ChannelSelect[5]=Turbine.UI.Lotro.CheckBox();
	self.ChannelSelect[5]:SetParent(self)
	self.ChannelSelect[5]:SetText(Resource[language][18]);
	self.ChannelSelect[5]:SetSize(offset-10,20);
	self.ChannelSelect[5]:SetPosition(10,self.ChannelCaption:GetTop()+50);
	self.ChannelSelect[5].Value=Turbine.ChatType.Combat
	self.ChannelSelect[6]=Turbine.UI.Lotro.CheckBox();
	self.ChannelSelect[6]:SetParent(self)
	self.ChannelSelect[6]:SetText(Resource[language][19]);
	self.ChannelSelect[6]:SetSize(offset-10,20);
	self.ChannelSelect[6]:SetPosition(offset,self.ChannelCaption:GetTop()+50);
	self.ChannelSelect[6].Value=Turbine.ChatType.Death
	self.ChannelSelect[7]=Turbine.UI.Lotro.CheckBox();
	self.ChannelSelect[7]:SetParent(self)
	self.ChannelSelect[7]:SetText(Resource[language][20]);
	self.ChannelSelect[7]:SetSize(offset-10,20);
	self.ChannelSelect[7]:SetPosition(offset*2,self.ChannelCaption:GetTop()+50);
	self.ChannelSelect[7].Value=Turbine.ChatType.Emote
	self.ChannelSelect[8]=Turbine.UI.Lotro.CheckBox();
	self.ChannelSelect[8]:SetParent(self)
	self.ChannelSelect[8]:SetText(Resource[language][21]);
	self.ChannelSelect[8]:SetSize(offset-10,20);
	self.ChannelSelect[8]:SetPosition(offset*3,self.ChannelCaption:GetTop()+50);
	self.ChannelSelect[8].Value=Turbine.ChatType.Error
	self.ChannelSelect[9]=Turbine.UI.Lotro.CheckBox();
	self.ChannelSelect[9]:SetParent(self)
	self.ChannelSelect[9]:SetText(Resource[language][22]);
	self.ChannelSelect[9]:SetSize(offset-10,20);
	self.ChannelSelect[9]:SetPosition(10,self.ChannelCaption:GetTop()+75);
	self.ChannelSelect[9].Value=Turbine.ChatType.Fellowship
	self.ChannelSelect[10]=Turbine.UI.Lotro.CheckBox();
	self.ChannelSelect[10]:SetParent(self)
	self.ChannelSelect[10]:SetText(Resource[language][23]);
	self.ChannelSelect[10]:SetSize(offset-10,20);
	self.ChannelSelect[10]:SetPosition(offset,self.ChannelCaption:GetTop()+75);
	self.ChannelSelect[10].Value=Turbine.ChatType.Kinship
	self.ChannelSelect[11]=Turbine.UI.Lotro.CheckBox();
	self.ChannelSelect[11]:SetParent(self)
	self.ChannelSelect[11]:SetText(Resource[language][24]);
	self.ChannelSelect[11]:SetSize(offset-10,20);
	self.ChannelSelect[11]:SetPosition(offset*2,self.ChannelCaption:GetTop()+75);
	self.ChannelSelect[11].Value=Turbine.ChatType.LFF
	self.ChannelSelect[12]=Turbine.UI.Lotro.CheckBox();
	self.ChannelSelect[12]:SetParent(self)
	self.ChannelSelect[12]:SetText(Resource[language][25]);
	self.ChannelSelect[12]:SetSize(offset-10,20);
	self.ChannelSelect[12]:SetPosition(offset*3,self.ChannelCaption:GetTop()+75);
	self.ChannelSelect[12].Value=Turbine.ChatType.Officer
	self.ChannelSelect[13]=Turbine.UI.Lotro.CheckBox();
	self.ChannelSelect[13]:SetParent(self)
	self.ChannelSelect[13]:SetText(Resource[language][26]);
	self.ChannelSelect[13]:SetSize(offset-10,20);
	self.ChannelSelect[13]:SetPosition(10,self.ChannelCaption:GetTop()+100);
	self.ChannelSelect[13].Value=Turbine.ChatType.OOC
	self.ChannelSelect[14]=Turbine.UI.Lotro.CheckBox();
	self.ChannelSelect[14]:SetParent(self)
	self.ChannelSelect[14]:SetText(Resource[language][27]);
	self.ChannelSelect[14]:SetSize(offset-10,20);
	self.ChannelSelect[14]:SetPosition(offset,self.ChannelCaption:GetTop()+100);
	self.ChannelSelect[14].Value=Turbine.ChatType.Quest
	self.ChannelSelect[15]=Turbine.UI.Lotro.CheckBox();
	self.ChannelSelect[15]:SetParent(self)
	self.ChannelSelect[15]:SetText(Resource[language][28]);
	self.ChannelSelect[15]:SetSize(offset-10,20);
	self.ChannelSelect[15]:SetPosition(offset*2,self.ChannelCaption:GetTop()+100);
	self.ChannelSelect[15].Value=Turbine.ChatType.Raid
	self.ChannelSelect[16]=Turbine.UI.Lotro.CheckBox();
	self.ChannelSelect[16]:SetParent(self)
	self.ChannelSelect[16]:SetText(Resource[language][29]);
	self.ChannelSelect[16]:SetSize(offset-10,20);
	self.ChannelSelect[16]:SetPosition(offset*3,self.ChannelCaption:GetTop()+100);
	self.ChannelSelect[16].Value=Turbine.ChatType.Regional

	self.ChannelSelect[17]=Turbine.UI.Lotro.CheckBox();
	self.ChannelSelect[17]:SetParent(self)
	self.ChannelSelect[17]:SetText(Resource[language][30]);
	self.ChannelSelect[17]:SetSize(offset-10,20);
	self.ChannelSelect[17]:SetPosition(10,self.ChannelCaption:GetTop()+125);
	self.ChannelSelect[17].Value=Turbine.ChatType.Roleplay
	self.ChannelSelect[18]=Turbine.UI.Lotro.CheckBox();
	self.ChannelSelect[18]:SetParent(self)
	self.ChannelSelect[18]:SetText(Resource[language][31]);
	self.ChannelSelect[18]:SetSize(offset-10,20);
	self.ChannelSelect[18]:SetPosition(offset,self.ChannelCaption:GetTop()+125);
	self.ChannelSelect[18].Value=Turbine.ChatType.Say
	self.ChannelSelect[19]=Turbine.UI.Lotro.CheckBox();
	self.ChannelSelect[19]:SetParent(self)
	self.ChannelSelect[19]:SetText(Resource[language][32]);
	self.ChannelSelect[19]:SetSize(offset-10,20);
	self.ChannelSelect[19]:SetPosition(offset*2,self.ChannelCaption:GetTop()+125);
	self.ChannelSelect[19].Value=Turbine.ChatType.Standard
	self.ChannelSelect[20]=Turbine.UI.Lotro.CheckBox();
	self.ChannelSelect[20]:SetParent(self)
	self.ChannelSelect[20]:SetText(Resource[language][33]);
	self.ChannelSelect[20]:SetSize(offset-10,20);
	self.ChannelSelect[20]:SetPosition(offset*3,self.ChannelCaption:GetTop()+125);
	self.ChannelSelect[20].Value=Turbine.ChatType.Tell
	self.ChannelSelect[21]=Turbine.UI.Lotro.CheckBox();
	self.ChannelSelect[21]:SetParent(self)
	self.ChannelSelect[21]:SetText(Resource[language][34]);
	self.ChannelSelect[21]:SetSize(offset-10,20);
	self.ChannelSelect[21]:SetPosition(10,self.ChannelCaption:GetTop()+150);
	self.ChannelSelect[21].Value=Turbine.ChatType.Trade
	self.ChannelSelect[22]=Turbine.UI.Lotro.CheckBox();
	self.ChannelSelect[22]:SetParent(self)
	self.ChannelSelect[22]:SetText(Resource[language][35]);
	self.ChannelSelect[22]:SetSize(offset-10,20);
	self.ChannelSelect[22]:SetPosition(offset,self.ChannelCaption:GetTop()+150);
	self.ChannelSelect[22].Value=Turbine.ChatType.Tribe
	self.ChannelSelect[23]=Turbine.UI.Lotro.CheckBox();
	self.ChannelSelect[23]:SetParent(self)
	self.ChannelSelect[23]:SetText(Resource[language][36]);
	self.ChannelSelect[23]:SetSize(offset-10,20);
	self.ChannelSelect[23]:SetPosition(offset*2,self.ChannelCaption:GetTop()+150);
	self.ChannelSelect[23].Value=Turbine.ChatType.Unfiltered
	self.ChannelSelect[24]=Turbine.UI.Lotro.CheckBox();
	self.ChannelSelect[24]:SetParent(self)
	self.ChannelSelect[24]:SetText(Resource[language][37]);
	self.ChannelSelect[24]:SetSize(offset-10,20);
	self.ChannelSelect[24]:SetPosition(offset*3,self.ChannelCaption:GetTop()+150);
	self.ChannelSelect[24].Value=Turbine.ChatType.UserChat1
	self.ChannelSelect[25]=Turbine.UI.Lotro.CheckBox();
	self.ChannelSelect[25]:SetParent(self)
	self.ChannelSelect[25]:SetText(Resource[language][38]);
	self.ChannelSelect[25]:SetSize(offset-10,20);
	self.ChannelSelect[25]:SetPosition(10,self.ChannelCaption:GetTop()+175);
	self.ChannelSelect[25].Value=Turbine.ChatType.UserChat2
	self.ChannelSelect[26]=Turbine.UI.Lotro.CheckBox();
	self.ChannelSelect[26]:SetParent(self)
	self.ChannelSelect[26]:SetText(Resource[language][39]);
	self.ChannelSelect[26]:SetSize(offset-10,20);
	self.ChannelSelect[26]:SetPosition(offset,self.ChannelCaption:GetTop()+175);
	self.ChannelSelect[26].Value=Turbine.ChatType.UserChat3
	self.ChannelSelect[27]=Turbine.UI.Lotro.CheckBox();
	self.ChannelSelect[27]:SetParent(self)
	self.ChannelSelect[27]:SetText(Resource[language][40]);
	self.ChannelSelect[27]:SetSize(offset-10,20);
	self.ChannelSelect[27]:SetPosition(offset*2,self.ChannelCaption:GetTop()+175);
	self.ChannelSelect[27].Value=Turbine.ChatType.UserChat4
--	self.ChannelSelect[28]=Turbine.UI.Lotro.CheckBox();
--	self.ChannelSelect[28]:SetParent(self)
--	self.ChannelSelect[28]:SetText(Resource[language][25]);
--	self.ChannelSelect[28]:SetSize(offset-10,20);
--	self.ChannelSelect[28]:SetPosition(offset*3,self.ChannelCaption:GetTop()+175);
--	self.ChannelSelect[28].Value=Turbine.ChatType.Officer

	self.PatternCaption=Turbine.UI.Label();
	self.PatternCaption:SetParent(self);
	self.PatternCaption:SetSize(100,20);
	self.PatternCaption:SetPosition(10,self.ChannelCaption:GetTop()+200);
	self.PatternCaption:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft);
	self.PatternCaption:SetText(Resource[language][13]..":");
	self.PatternText=Turbine.UI.Lotro.TextBox();
	self.PatternText:SetParent(self);
	self.PatternText:SetSize(self:GetWidth()-self.PatternCaption:GetLeft()-self.PatternCaption:GetWidth()-12,20);
	self.PatternText:SetFont(fontFace);
	self.PatternText:SetPosition(self.PatternCaption:GetLeft()+self.PatternCaption:GetWidth()+2,self.PatternCaption:GetTop());

	self.SaveButton=Turbine.UI.Lotro.Button();
	self.SaveButton:SetParent(self);
	self.SaveButton:SetSize(100,20);
	self.SaveButton:SetText(Resource[language][41]);
	self.SaveButton:SetPosition(20,self:GetHeight()-45);
	self.SaveButton.MouseClick=function()
		-- if the alertselect is on New, add the definition to the Alerts, otherwise update the one pointed to by the select list
		local tmpIndex;
		local tmpAlertIndex=self.AlertSelect:GetValue();
		if tmpAlertIndex==0 then
			tmpAlertIndex=#Alerts+1
			-- need to add to the alerts list
			self.AlertSelect:AddItem(self.LabelText:GetText(),tmpAlertIndex);
			Alerts[tmpAlertIndex]={};
		else
			-- update the alertSelect label
			self.AlertSelect.CurrentValue:SetText(self.LabelText:GetText());
			self.AlertSelect.ListData:GetItem(self.AlertSelect:GetSelectedIndex()):SetText(self.LabelText:GetText());
		end
		Alerts[tmpAlertIndex][1]=self.LabelText:GetText();
		Alerts[tmpAlertIndex][2]=self.MessageText:GetText();
		Alerts[tmpAlertIndex][3]=tonumber(self.DurationText:GetText());
		Alerts[tmpAlertIndex][4]=self.MessageColor:GetColor();
		Alerts[tmpAlertIndex][5]=self.FontSelect:GetValue();
		Alerts[tmpAlertIndex][6]=tonumber(self.IntervalText:GetText());
		Alerts[tmpAlertIndex][7]={};
		local m,tmpVal;
		for tmpIndex=1,#self.ChannelSelect do
			m,tmpVal=math.frexp(self.ChannelSelect[tmpIndex].Value);

			Alerts[tmpAlertIndex][7][tmpVal]=self.ChannelSelect[tmpIndex]:IsChecked()
		end
		Alerts[tmpAlertIndex][8]=self.PatternText:GetText();
	end

	self.TestButton=Turbine.UI.Lotro.Button();
	self.TestButton:SetParent(self);
	self.TestButton:SetSize(100,20);
	self.TestButton:SetText(Resource[language][42]);
	self.TestButton:SetPosition((self:GetWidth()-self.TestButton:GetWidth())/2,self:GetHeight()-45);
	self.TestButton.MouseClick=function()
		self.Flasher.SetAlert(self.MessageText:GetText(), tonumber(self.DurationText:GetText()), self.MessageColor:GetColor(), self.FontSelect:GetValue(), tonumber(self.IntervalText:GetText()));
	end

	self.KeyDown=function(sender, args)
		if ( args.Action == Turbine.UI.Lotro.Action.Escape ) then
			self:SetVisible(false);
		end
		if ( args.Action == 268435635 ) then -- toggle HUD
			alertHUDState = not alertHUDState;
			if alertHUDState then
				alertMain:SetVisible(alertMainState);
			else
				alertMainState=alertMain:IsVisible();
				alertMain:SetVisible(false);
			end
		end
	end
	self.loaded=false;
	self:SetWantsUpdates(true);
	self.Update=function()
		if not self.loaded then
			self.loaded=true;
			Plugins["Alerter"].Unload = function(self,sender,args)
				UnloadPlugin();
			end
			self:SetWantsUpdates(false);
		end
	end

	self.RemoveShellCommands=function(sender,args)
		Turbine.Shell.RemoveCommand(self.shellCmd)
	end

	self.shellCmd=Turbine.ShellCommand();
	local numberOfCommandsRegistered = Turbine.Shell.AddCommand("Alerter",self.shellCmd);

	self.shellCmd.Execute = function(sender, cmd, args)
		if (string.lower(cmd) == "alerter") then
			local pos1=string.find(args," ",0,true);
			local command;
			if pos1~=nil and pos1>0 then
				command=string.lower(string.sub(args,1,pos1-1));
			else
				command=string.lower(args);
			end
			if (command=="setup") then
				self:SetVisible(true);
			else
				Turbine.Shell.WriteLine("usage:");
				Turbine.Shell.WriteLine("  /Alerter setup");
				Turbine.Shell.WriteLine("       Displays the Alerter Setup window.");
			end
		end
	end
	self.shellCmd.GetHelp = function(sender, cmd)
		return("usage:\n  /Alerter setup\n       Displays the Alerter Setup window.");
	end

	self.shellCmd.GetShortHelp = function(sender, cmd)
		return("Alerter - usage: /Alerter setup");
	end
	self:SetWantsKeyEvents(true);

	Turbine.Chat.Received=function(sender, args)
		local channelMatched;
		local m,tmpChannel,tmpIndex;
		for tmpIndex=1,#Alerts do
			channelMatched=false;
			if Alerts[tmpIndex][7]~=nil and type(Alerts[tmpIndex][7])=="table" then
				if Alerts[tmpIndex][7][0]~=nil and Alerts[tmpIndex][7][0] then
					channelMatched=true;
				else
					m,tmpChannel=math.frexp(args.ChatType);
					if Alerts[tmpIndex][7][tmpChannel]~=nil and Alerts[tmpIndex][7][tmpChannel] then
						channelMatched=true;
					end
				end
			end
			-- channel matched
-- compare to pattern
			if channelMatched then
				if Alerts[tmpIndex][8]==nil or string.find(string.lower(args.Message),string.lower(Alerts[tmpIndex][8])) then
					local msg;
--					Turbine.Shell.WriteLine("Channel:["..args.ChatType.."] Sender:"..args.Sender.." Message:"..args.Message)
					if Alerts[tmpIndex][2]==nil or Alerts[tmpIndex][2]=="" then
						msg=args.Message
					else
						msg=Alerts[tmpIndex][2]
					end
					alertMain.Flasher.SetAlert(msg, Alerts[tmpIndex][3], Alerts[tmpIndex][4], Alerts[tmpIndex][5], Alerts[tmpIndex][6]);
				end
			end
		end
		-- args.ChatType
		-- args.Sender
		-- args.Message
	end
end
alertMain=Alerter()
